@@ -35,7 +35,9 @@
 ;;; Code:
 ;; * 代码                                                                 :code:
 (require 'pyim-common)
+(require 'pyim-dict)
 (require 'subr-x)
+(require 'pyim-dcache)
 
 (defvar pyim-dregcache-partition-minimum-size 32
   "小于这个数值(单位为M)的词典不需要用分区算法加速.可节约一半的内存.")
@@ -45,6 +47,20 @@
 (defvar pyim-dregcache-iword2count nil)
 (defvar pyim-dregcache-dicts-md5 nil)
 
+(defun pyim-dregcache-update (&optional force)
+  "读取并加载所有相关词库 dcache.
+
+如果 FORCE 为真，强制加载。"
+  (pyim-dregcache-init-variables)
+  (when pyim-dcache-auto-update
+    (pyim-dregcache-update-personal-words force)
+    (let* ((dict-files (mapcar (lambda (x)
+                                 (unless (plist-get x :disable)
+                                   (plist-get x :file)))
+                               `(,@pyim-dicts ,@pyim-extra-dicts)))
+           (dicts-md5 (pyim-dcache-create-files-md5 dict-files)))
+      (pyim-dregcache-update-code2word dict-files dicts-md5 force))))
+
 (defun pyim-dregcache-variable-file (variable)
   "Get VARIABLE dcache file path."
   (concat (file-name-as-directory pyim-dcache-directory)
@@ -68,15 +84,14 @@
         (buffer-string)))))
 
 (defun pyim-dregcache-sort-words (words-list)
-  "对 WORDS-LIST 排序，词频大的排在前面.
-
-排序使用 `pyim-dregcache-iword2count' 中记录的词频信息"
-  (sort words-list
-        (lambda (a b)
-          (let ((a (car (split-string a ":")))
-                (b (car (split-string b ":"))))
-            (> (or (gethash a pyim-dregcache-iword2count) 0)
-               (or (gethash b pyim-dregcache-iword2count) 0))))))
+  "对 WORDS-LIST 排序，词频大的排在前面."
+  (let ((iword2count pyim-dregcache-iword2count))
+    (sort words-list
+          (lambda (a b)
+            (let ((a (car (split-string a ":")))
+                  (b (car (split-string b ":"))))
+              (> (or (gethash a iword2count) 0)
+                 (or (gethash b iword2count) 0)))))))
 
 (defun pyim-dregcache-sort-icode2word ()
   "对个人词库排序."
@@ -89,7 +104,7 @@
         ;; ((string-match "^\\([a-z]+ \\|[a-z]+-[a-z]+ \\|[a-z]+-[a-z]+-[a-z]+ \\)\\(.*\\)" l)
         (let* ((pinyin (match-string 1 l))
                (words (pyim-dregcache-sort-words (split-string (match-string 2 l) " "))))
-          (insert (format "%s\n" (concat pinyin (mapconcat #'identity words " "))))))
+          (insert (format "%s\n" (concat pinyin (string-join words " "))))))
        ;; 其他词
        ((string= l "")
         ;; skip empty line
@@ -268,21 +283,25 @@ DICT-FILES 是词库文件列表. DICTS-MD5 是词库的MD5校验码.
 
 (defun pyim-dregcache-get (code &optional from)
   "从 `pyim-dregcache-cache' 搜索 CODE, 得到对应的词条."
-  (if (or (memq 'icode2word from)
-          (memq 'ishortcode2word from))
-      (pyim-dregcache-get-icode2word-ishortcode2word code)
-    (let ((dict-files (pyim-dregcache-all-dict-files))
-          result)
-      (when pyim-debug (message "pyim-dregcache-get is called. code=%s" code))
-      (when dict-files
-        (dolist (file dict-files)
-          (let* ((file-info (lax-plist-get pyim-dregcache-cache file))
-                 (content (pyim-dregcache-get-content code file-info)))
-            (setq result (append (pyim-dregcache-get-1 content code) result)))))
-      ;; `push' plus `nreverse' is more efficient than `add-to-list'
-      ;; Many examples exist in Emacs' own code
-      (setq result (nreverse result))
-      `(,@result ,@(pyim-pymap-py2cchar-get code t t)))))
+  (cond ((or (memq 'icode2word from)
+             (memq 'ishortcode2word from))
+         (pyim-dregcache-get-icode2word-ishortcode2word code))
+        ;; FIXME: pyim-dregcache 暂时不支持 iword2count-recent-10-words 和
+        ;; iword2count-recent-50-words.
+        ((or (memq 'iword2count-recent-10-words from)
+             (memq 'iword2count-recent-50-words from))
+         nil)
+        (t (let ((dict-files (pyim-dregcache-all-dict-files))
+                 result)
+             (when pyim-debug (message "pyim-dregcache-get is called. code=%s" code))
+             (when dict-files
+               (dolist (file dict-files)
+                 (let* ((file-info (lax-plist-get pyim-dregcache-cache file))
+                        (content (pyim-dregcache-get-content code file-info)))
+                   (setq result (append (pyim-dregcache-get-1 content code) result)))))
+             ;; `push' plus `nreverse' is more efficient than `add-to-list'
+             ;; Many examples exist in Emacs' own code
+             (nreverse result)))))
 
 (defun pyim-dregcache-get-icode2word-ishortcode2word (code)
   "以 CODE 搜索个人词和个人联想词.  正则表达式搜索词库,不需要为联想词开单独缓存."
@@ -329,13 +348,12 @@ DICT-FILES 是词库文件列表. DICTS-MD5 是词库的MD5校验码.
 
 (defun pyim-dregcache-init-variables ()
   "初始化 cache 缓存相关变量."
-  (pyim-dcache-set-variable
-   'pyim-dregcache-iword2count
-   nil
+  (pyim-dcache-init-variable
+   pyim-dregcache-iword2count
    ;; dregcache 引擎也需要词频信息，第一次使用 dregcache 引擎的时候，
    ;; 自动导入 dhashcache 引擎的词频信息，以后两个引擎的词频信息就
    ;; 完全分开了。
-   (pyim-dcache-get-variable 'pyim-dhashcache-iword2count))
+   (pyim-dcache-get-value 'pyim-dhashcache-iword2count))
   (unless pyim-dregcache-icode2word
     (pyim-dregcache-update-personal-words t)))
 
@@ -344,25 +362,29 @@ DICT-FILES 是词库文件列表. DICTS-MD5 是词库的MD5校验码.
   (when pyim-debug (message "pyim-dregcache-save-personal-dcache-to-file called"))
   ;; 用户选择过的词存为标准辞典格式保存
   (when pyim-dregcache-icode2word
-    (pyim-dregcache-save-variable 'pyim-dregcache-icode2word
-                                  pyim-dregcache-icode2word))
+    (pyim-dregcache-save-variable
+     'pyim-dregcache-icode2word
+     pyim-dregcache-icode2word))
   ;; 词频
-  (pyim-dcache-save-variable 'pyim-dregcache-iword2count))
+  (pyim-dcache-save-variable
+   'pyim-dregcache-iword2count
+   pyim-dregcache-iword2count))
 
-(defun pyim-dregcache-insert-export-content ()
+(defun pyim-dregcache-export-words-and-counts ()
   "TODO"
   )
 
-(defun pyim-dregcache-update-iword2count (word &optional _prepend wordcount-handler)
+(defun pyim-dregcache-update-iword2count (word &optional wordcount-handler)
   "保存词频到缓存."
   (when pyim-debug (message "pyim-dregcache-update-iword2count. word=%s" word))
-  (let* ((orig-value (gethash word pyim-dregcache-iword2count))
+  (let* ((orig-value
+          (or (gethash word pyim-dregcache-iword2count) 0))
          (new-value (cond
                      ((functionp wordcount-handler)
                       (funcall wordcount-handler orig-value))
                      ((numberp wordcount-handler)
                       wordcount-handler)
-                     (t (+ (or orig-value 0) 1)))))
+                     (t (+ orig-value 1)))))
     (unless (equal orig-value new-value)
       (puthash word new-value pyim-dregcache-iword2count))))
 
