@@ -129,18 +129,18 @@
 (defun pyim-page-preview-create:rime (&optional separator)
   (let* ((preedit (or (liberime-get-preedit)
                       (pyim-entered-get 'point-before))))
-    (pyim-with-entered-buffer
-      (if (equal 1 (point))
-          (concat "|" preedit)
-        (concat (replace-regexp-in-string (concat separator "'") "'" preedit)
-                " |" (buffer-substring-no-properties (point) (point-max)))))))
+    (pyim-process-with-entered-buffer
+     (if (equal 1 (point))
+         (concat "|" preedit)
+       (concat (replace-regexp-in-string (concat separator "'") "'" preedit)
+               " |" (buffer-substring-no-properties (point) (point-max)))))))
 
 (defvar pyim-liberime-code-log nil)
 (defvar pyim-liberime-word-log nil)
 (defun pyim-select-word:rime ()
   "从选词框中选择当前词条，然后删除该词条对应拼音。"
   (interactive)
-  (pyim-outcome-handle 'candidate)
+  (pyim-process-outcome-handle 'candidate)
   (let* ((entered (pyim-entered-get 'point-before))
          (word (string-remove-prefix
                 (or (pyim-outcome-get 1) "") (pyim-outcome-get)))
@@ -152,11 +152,11 @@
     (if (or (> (length to-be-translated) 0) ;是否有光标前未转换的字符串
             (> (length (pyim-entered-get 'point-after)) 0)) ;是否有光标后字符串
         (progn
-          (pyim-with-entered-buffer
+          (pyim-process-with-entered-buffer
             (delete-region (point-min) (point))
             (insert to-be-translated)
             (goto-char (point-max)))
-          (pyim-entered-refresh))
+          (pyim-process-run))
       ;; 在 rime 后端造词和调整瓷瓶词频
       (pyim-liberime-create-word
        (reverse pyim-liberime-code-log)
@@ -164,11 +164,11 @@
       ;; 使用 rime 的同时，也附带的优化 quanpin 的词库。
       (let ((pyim-default-scheme 'quanpin))
         (if (member (pyim-outcome-get) pyim-candidates)
-            (pyim-create-pyim-word (pyim-outcome-get) t)
-          (pyim-create-pyim-word (pyim-outcome-get))))
+            (pyim-process-create-word (pyim-outcome-get) t)
+          (pyim-process-create-word (pyim-outcome-get))))
       (setq pyim-liberime-code-log nil)
       (setq pyim-liberime-word-log nil)
-      (pyim-terminate-translation)
+      (pyim-process-terminate)
       ;; pyim 使用这个 hook 来处理联想词。
       (run-hooks 'pyim-select-finish-hook))))
 
@@ -190,7 +190,7 @@
           `(:select current :replace-with ,commit))
          (t nil))))))
 
-(add-to-list 'pyim-autoselector 'pyim-autoselector-rime)
+(add-to-list 'pyim-process-autoselector 'pyim-autoselector-rime)
 
 (defun pyim-liberime-create-word (codes words)
   "通过 CODES 和 WORDS 的信息，在 rime 后端重新造词和调整词频。
@@ -205,7 +205,7 @@
              (not (cl-find-if-not #'stringp codes))
              (not (cl-find-if-not #'stringp words)))
     (liberime-clear-composition)
-    (dolist (key (string-to-list (mapconcat #'identity codes "")))
+    (dolist (key (string-to-list (string-join codes)))
       (liberime-process-key key))
     (let (word)
       (while (setq word (pop words))
@@ -225,22 +225,6 @@
                 (setq words nil))
                (t (liberime-process-key 65366))))))))))
 
-(defun pyim-create-rime-word (word &optional _prepend _wordcount-handler)
-  "Create WORD at current rime backend.
-ONlY works with quanpin."
-  ;; 判断当前 rime 环境是否支持全拼，如果支持，就添加词条。
-  (ignore-errors
-    (let ((codes (pyim-cstring-to-codes word 'quanpin)))
-      (when (member "你好" (liberime-search "nihao" 10))
-        (dolist (code codes)
-          (unless (pyim-string-match-p "[^ a-z-]" code)
-            (pyim-liberime-create-word
-             (split-string code "-")
-             (remove "" (split-string word "")))
-            (pyim-terminate-translation:rime)))))))
-
-(advice-add 'pyim-create-word :after #'pyim-create-rime-word)
-
 (defun pyim-liberime-get-code (word input &optional _limit)
   "Get the code of WORD from the beginning of INPUT.
 `liberime-search' with LIMIT argument is used internal.
@@ -252,13 +236,12 @@ Please see: https://github.com/rime/librime/issues/349"
   (cond
    ;; 处理基于语音的输入法，比如：拼音，这类输入法 preedit 一般用空格
    ;; 分隔，与汉字一一对应。
-   ((string-match-p
-     (mapconcat #'identity
-                '("pinyin" "luna" "terra" "bopomofo" "stenotype"
-                  "jyut6ping3" "wugniu" "soutzoe" "zyenpheng"
-                  "sampheng" "clover")
-                "\\|")
-     (alist-get 'schema_id (liberime-get-status)))
+   ((string-match-p (string-join
+                     '("pinyin" "luna" "terra" "bopomofo" "stenotype"
+                       "jyut6ping3" "wugniu" "soutzoe" "zyenpheng"
+                       "sampheng" "clover")
+                     "\\|")
+                    (alist-get 'schema_id (liberime-get-status)))
     (unless (liberime-get-preedit)
       (liberime-search input 1))
     (let* ((n (length word))
@@ -273,11 +256,8 @@ Please see: https://github.com/rime/librime/issues/349"
             (setq i 0)
           (setq i (- i 1))))
       str))
-   ((string-match-p
-     (mapconcat #'identity
-                '("wubi86" "wubi98")
-                "\\|")
-     (alist-get 'schema_id (liberime-get-status)))
+   ((string-match-p (string-join '("wubi86" "wubi98") "\\|")
+                    (alist-get 'schema_id (liberime-get-status)))
     (let ((lst (split-string (liberime-get-preedit) "[ ']+"))
           (str "")
           words)
@@ -290,7 +270,7 @@ Please see: https://github.com/rime/librime/issues/349"
    ;; 找不到通用的处理方式的话就不做截取处理。
    (t input)))
 
-(defun pyim-terminate-translation:rime ()
+(defun pyim-process-terminate:rime ()
   (liberime-clear-commit)
   (liberime-clear-composition))
 
